import CssMinimizerPlugin from 'css-minimizer-webpack-plugin';
import TerserPlugin from 'terser-webpack-plugin';

function optimizationConfig(api) {
  api.injectWebpackConfig((config) => {
    if (api.mode === "development")
      return;
    config.optimization.minimizer("css").use(CssMinimizerPlugin, [
      {
        parallel: true,
        minify: CssMinimizerPlugin.cssnanoMinify,
        minimizerOptions: {
          preset: [
            "default",
            {
              mergeLonghand: false,
              cssDeclarationSorter: false
            }
          ]
        }
      }
    ]).end();
    config.optimization.minimize(true).minimizer("terser").use(TerserPlugin, [
      {
        minify: TerserPlugin.terserMinify,
        extractComments: false,
        terserOptions: {
          compress: {
            // turn off flags with small gains to speed up minification
            arrows: false,
            collapse_vars: false,
            // 0.3kb
            comparisons: false,
            computed_props: false,
            hoist_funs: false,
            hoist_props: false,
            hoist_vars: false,
            inline: false,
            loops: false,
            negate_iife: false,
            properties: false,
            reduce_funcs: false,
            reduce_vars: false,
            switches: false,
            toplevel: false,
            typeofs: false,
            // a few flags with noticeable gains/speed ratio
            // numbers based on out of the box vendor bundle
            booleans: true,
            // 0.7kb
            if_return: true,
            // 0.4kb
            sequences: true,
            // 0.7kb
            unused: true,
            // 2.3kb
            // required features to drop conditional branches
            conditionals: true,
            dead_code: true,
            evaluate: true
          }
        }
      }
    ]).end();
    config.optimization.splitChunks({
      cacheGroups: {
        defaultVendors: {
          name: `chunk-vendors`,
          test: /[\\/]node_modules[\\/]/,
          priority: -10,
          chunks: "initial"
        },
        common: {
          name: `chunk-common`,
          minChunks: 2,
          priority: -20,
          chunks: "initial",
          reuseExistingChunk: true
        }
      }
    }).end();
    config.performance.hints("warning").maxAssetSize(25e4).maxEntrypointSize(4e5).end();
  });
}

export { optimizationConfig as default };
