'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs-extra');
var require$$1 = require('path');
var zlib = require('zlib');
var index$1 = require('./node_modules/.pnpm/chalk@5.2.0/node_modules/chalk/source/index.js');
var index = require('./node_modules/.pnpm/cliui@8.0.1/node_modules/cliui/index.js');

function formatStats(stats, dir, api) {
  const json = stats.toJson({
    hash: false,
    modules: false,
    chunks: false
  });
  let assets = json.assets ? json.assets : json?.children?.reduce((acc, child) => acc.concat(child.assets), []);
  const seenNames = /* @__PURE__ */ new Map();
  const isJS = (val) => /\.js$/.test(val || "");
  const isCSS = (val) => /\.css$/.test(val || "");
  const isMinJS = (val) => /\.min\.js$/.test(val || "");
  assets = assets?.map((a) => {
    a.name = a?.name?.split("?")[0];
    return a;
  }).filter((a) => {
    if (seenNames.has(a.name)) {
      return false;
    }
    seenNames.set(a.name, true);
    return isJS(a?.name) || isCSS(a?.name);
  }).sort(
    (a, b) => {
      if (isJS(a.name) && isCSS(b.name))
        return -1;
      if (isCSS(a.name) && isJS(b.name))
        return 1;
      if (isMinJS(a.name) && !isMinJS(b.name))
        return -1;
      if (!isMinJS(a.name) && isMinJS(b.name))
        return 1;
      return b.size - a.size;
    }
  );
  function formatSize(size) {
    return (size / 1024).toFixed(2) + " KiB";
  }
  function getGzippedSize(asset) {
    const filepath = api.resolve(require$$1.join(dir, asset?.name));
    const buffer = fs.readFileSync(filepath);
    return formatSize(zlib.gzipSync(buffer).length);
  }
  function makeRow(a, b, c) {
    return `  ${a}	    ${b}	 ${c}`;
  }
  const ui = index.default({ width: process.stdout.columns || 100 });
  ui.div(
    makeRow(
      index$1.default.cyan.bold(`File`),
      index$1.default.cyan.bold(`Size`),
      index$1.default.cyan.bold(`Gzipped`)
    ) + `

` + assets?.map(
      (asset) => makeRow(
        /js$/.test(asset?.name) ? index$1.default.green(require$$1.join(dir, asset?.name)) : index$1.default.blue(require$$1.join(dir, asset?.name)),
        formatSize(asset.size),
        getGzippedSize(asset)
      )
    ).join(`
`)
  );
  const time = stats.endTime - stats.startTime;
  const now = (/* @__PURE__ */ new Date()).toISOString();
  const hash = stats.hash;
  const info = `\u65F6\u95F4: ${index$1.default.white(now)} - Hash: ${index$1.default.white(
    hash
  )} - \u8017\u65F6: ${index$1.default.white(time)}ms`;
  return `${ui.toString()}

  ${index$1.default.gray(
    `\u7701\u7565\u56FE\u50CF\u548C\u5176\u4ED6\u7C7B\u578B\u7684\u8D44\u4EA7\u3002`
  )}

  ${info}
`;
}

exports.default = formatStats;
