'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var require$$1 = require('path');
var formatStats = require('./format-stats.js');
var WebpackDevServer = require('webpack-dev-server');
var webpack = require('webpack');
var log = require('./packages/shared/build/esm/log.js');
require('./packages/shared/build/esm/loading.js');
require('./packages/shared/build/esm/node_modules/.pnpm/semver@7.5.1/node_modules/semver/index.js');
require('./packages/shared/build/esm/node_modules/.pnpm/chalk@5.2.0/node_modules/chalk/source/index.js');
require('./packages/shared/build/esm/node_modules/.pnpm/fs-extra@11.1.1/node_modules/fs-extra/lib/index.js');
require('node:fs');
require('node:fs/promises');
require('node:path');
require('node:url');
require('./packages/shared/build/esm/node_modules/.pnpm/parse-json@7.0.0_typescript@5.0.4/node_modules/parse-json/index.js');
require('./packages/shared/build/esm/node_modules/.pnpm/normalize-package-data@5.0.0/node_modules/normalize-package-data/lib/normalize.js');
require('child_process');
require('./packages/shared/build/esm/node_modules/.pnpm/esbuild@0.17.18/node_modules/esbuild/lib/main.js');
require('module');
var base_config = require('./base.config.js');
var css_config = require('./css.config.js');
var asset_config = require('./asset.config.js');
var html_config = require('./html.config.js');
var optimization_config = require('./optimization.config.js');
var babel_config = require('./babel.config.js');
var cache_config = require('./cache.config.js');
var moduleFederation_config = require('./module-federation.config.js');
var devServer = require('./dev-server.js');

function webpackPlugin(api) {
  base_config.default(api);
  css_config.default(api);
  asset_config.default(api);
  optimization_config.default(api);
  babel_config.default(api);
  cache_config.default(api);
  moduleFederation_config.default(api);
  api.registerCommand(
    "build",
    {
      description: "\u4F7F\u7528 webpack \u6346\u7ED1\u9879\u76EE",
      usage: "adc build [entry] [\u53C2\u6570]",
      parameters: {
        "--mode": "\u6307\u5B9A\u73AF\u5883\u6A21\u5F0F (\u9ED8\u8BA4\u503C: production)",
        "--dest": "\u6307\u5B9A\u8F93\u51FA\u76EE\u5F55 (\u9ED8\u8BA4\u503C: dist)",
        "--target": "\u6307\u5B9A\u6784\u5EFA\u6A21\u677F app | lib (\u9ED8\u8BA4\u503C: app)"
      }
    },
    async (args) => {
      if (args.target !== "lib")
        html_config.default(api);
      const config = api.resolveWebpackConfig();
      await new Promise((resolve, reject) => {
        const webpackConfig = config.toConfig();
        webpack.webpack(webpackConfig, (err, stats) => {
          if (err) {
            return reject(err.message);
          }
          if (stats?.hasErrors()) {
            log.error(stats?.toString());
            return reject(new Error("\u6784\u5EFA\u5931\u8D25\uFF0C\u51FA\u73B0\u9519\u8BEF"));
          }
          log.info(
            formatStats.default(
              stats,
              require$$1.relative(api.cwd, webpackConfig.output?.path),
              api
            )
          );
          console.info("\u6784\u5EFA\u5B8C\u6210\uFF01");
          return resolve(true);
        });
      });
    }
  );
  api.registerCommand(
    "dev",
    {
      description: "\u4F7F\u7528 webpack-dev-server \u542F\u52A8\u4E00\u4E2A\u5F00\u53D1\u670D\u52A1\u5668",
      usage: "adc dev [\u53C2\u6570]",
      parameters: {
        "--open": "\u5728\u670D\u52A1\u5668\u542F\u52A8\u65F6\u6253\u5F00\u6D4F\u89C8\u5668",
        "--host": "\u6307\u5B9A host (\u9ED8\u8BA4\u503C: 0.0.0.0)",
        "--port": "\u6307\u5B9A port (\u9ED8\u8BA4\u503C: 8080)",
        "--mode": "\u6307\u5B9A\u73AF\u5883\u6A21\u5F0F (\u9ED8\u8BA4\u503C: development)"
      }
    },
    async (args) => {
      html_config.default(api);
      devServer.default(api);
      const config = api.resolveWebpackConfig();
      const compiler = webpack.webpack(config.toConfig());
      const server = new WebpackDevServer(
        {
          open: true,
          hot: true
        },
        compiler
      );
      return new Promise((resolve, reject) => {
        compiler.hooks.done.tap("adc serve", (stats) => {
          if (stats.hasErrors()) {
            return;
          }
          console.info();
          console.info(`  App running sucess`);
        });
        server.start().catch((err) => reject(err.message));
      });
    }
  );
}

exports.default = webpackPlugin;
